"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var R = require("ramda");
var definitions_interface_1 = require("../definitions.interface");
var definition_1 = require("../definition");
var cond_1 = require("./cond");
var array_1 = require("./array");
var entity_1 = require("./entity");
var type_1 = require("./type");
var nvl_1 = require("./nvl");
var clone_1 = require("./clone");
var wrapper_1 = require("./wrapper");
/**
 * @stable [30.10.2019]
 * @param {IGenericFieldEntity} config
 * @returns {AnyT}
 */
exports.buildFinalFieldValue = function (config) {
    var emptyValue = config.emptyValue, originalValue = config.originalValue, value = config.value;
    var finalOriginalValue = wrapper_1.isEmptyOriginalValueSet(config) ? emptyValue : originalValue;
    var result = type_1.isDef(finalOriginalValue) && R.equals(value, finalOriginalValue)
        ? definition_1.FIELD_VALUE_TO_CLEAR_DIRTY_CHANGES
        : value;
    return definition_1.FIELD_VALUE_TO_CLEAR_DIRTY_CHANGES === result
        ? (wrapper_1.isAlwaysReturnEmptyValueIfOriginalValue(config) ? emptyValue : result)
        : result;
};
/**
 * @stable [28.10.2019]
 * @param {IGenericFieldEntity} props
 * @returns {boolean}
 */
exports.isFieldInactive = function (props) {
    return wrapper_1.isDisabled(props) || wrapper_1.isReadOnly(props) || wrapper_1.inProgress(props);
};
/**
 * @stable [30.10.2019]
 * @param {IGenericFieldEntity} props
 * @returns {boolean}
 */
exports.isFieldNotModifiable = function (props) {
    return wrapper_1.isDisabled(props) || wrapper_1.isReadOnly(props) || !wrapper_1.isChangeable(props);
};
/**
 * @stable [30.10.2019]
 * @param {AnyT} value
 * @param {AnyT} emptyValue
 * @returns {boolean}
 */
exports.isFieldValuePresent = function (value, emptyValue) {
    return type_1.isDef(value) && !R.equals(value, emptyValue);
};
/**
 * @stable [27.08.2019]
 * @param {EntityIdT} key
 * @returns {string}
 */
exports.dynamicFieldName = function (key) { return "$$dynamicField-" + key; };
/**
 * @stable [28.08.2019]
 * @param {TEntity} object
 * @param {EntityIdT} key
 * @returns {AnyT}
 */
exports.dynamicFieldValue = function (object, key) {
    return Reflect.get(object, exports.dynamicFieldName(key));
};
/**
 * @stable [28.08.2019]
 * @param {TEntity[]} array
 * @param {(itm: TEntity) => EntityIdT} keyAccessor
 * @param {(itm: TEntity) => EntityIdT} valueAccessor
 * @returns {IKeyValue}
 */
exports.fromDynamicFieldsArray = function (array, keyAccessor, valueAccessor) {
    return R.mergeAll(array.map(function (itm) {
        var _a;
        return (_a = {},
            _a[exports.dynamicFieldName(keyAccessor(itm))] = valueAccessor(itm),
            _a);
    }));
};
/**
 * @stable [28.08.2019]
 * @param {EntityIdT[]} array
 * @param {(itm: EntityIdT) => EntityIdT} valueAccessor
 * @returns {IKeyValue}
 */
exports.fromDynamicFieldsIdsArray = function (array, valueAccessor) {
    return exports.fromDynamicFieldsArray(array, function (itm) { return itm; }, valueAccessor);
};
/**
 * @stable [13.10.2019]
 * @param {MultiFieldEntityT<TEntity extends IEntity>} entity
 * @returns {TEntity[]}
 */
exports.asMultiFieldEntities = function (entity) {
    var before = Date.now();
    if (R.isNil(entity)) {
        return definitions_interface_1.UNDEF;
    }
    if (entity_1.isNotMultiEntity(entity)) {
        return entity;
    }
    var multiEntity = entity;
    var sourceEntities = multiEntity.source || [];
    // Fill a cache
    var cachedSourceEntities = new Map();
    sourceEntities.forEach(function (itm) { return cachedSourceEntities.set(itm.id, itm); });
    // Pass a map to optimize
    var editedEntities = exports.asMultiFieldEditedEntities(entity, cachedSourceEntities);
    var cachedEditedEntities = new Map();
    if (array_1.isArrayNotEmpty(editedEntities)) {
        editedEntities.forEach(function (itm) { return cachedEditedEntities.set(itm.id, itm); });
    }
    // Remove the deleted entities
    multiEntity.remove.forEach(function (itm) { return cachedSourceEntities.delete(itm.id); });
    return multiEntity.add.concat(Array.from(cachedSourceEntities.values())
        .map(function (itm) { return nvl_1.nvl(cachedEditedEntities.get(itm.id), itm); }));
};
/**
 * @stable [12.10.2019]
 * @param {MultiFieldEntityT} entity
 * @param {Map<EntityIdT, TItem extends IEntity>} mappedSourcedItems
 * @returns {TItem[]}
 */
exports.asMultiFieldEditedEntities = function (entity, mappedSourcedItems) {
    var multiEntity = entity;
    if (R.isNil(entity)) {
        return definitions_interface_1.UNDEF;
    }
    if (entity_1.isNotMultiEntity(entity)) {
        return [];
    }
    var resultItems = new Map();
    var cachedSourceItems = mappedSourcedItems || new Map();
    if (R.isNil(mappedSourcedItems)) {
        multiEntity.source.forEach(function (sourceEntity) { return cachedSourceItems.set(sourceEntity.id, sourceEntity); });
    }
    multiEntity.edit.forEach(function (editedItem) {
        var editedItemId = editedItem.id;
        var cachedResultItem = resultItems.get(editedItemId);
        // Collect the changes
        var editedItem0 = cachedResultItem || clone_1.shallowClone(cachedSourceItems.get(editedItemId));
        editedItem0[editedItem.name] = editedItem.value;
        if (R.isNil(cachedResultItem)) {
            resultItems.set(editedItemId, editedItem0);
        }
    });
    return Array.from(resultItems.values());
};
/**
 * @stable [14.10.2019]
 * @param {NotMultiFieldEntityT} value
 * @returns {IEntity[]}
 */
exports.asEntitiesArray = function (value) {
    return type_1.isPrimitive(value) ? [{ id: value }] : value;
};
/**
 * @stable [14.10.2019]
 * @param {MultiFieldEntityT | EntityIdT} value
 * @returns {number}
 */
exports.asMultiFieldEntitiesLength = function (value) { return cond_1.ifNotNilThanValue(value, function () { return (entity_1.isNotMultiEntity(value)
    ? exports.asEntitiesArray(value)
    : exports.asMultiFieldEntities(value)).length; }, 0); };
/**
 * @stable [14.10.2019]
 * @param {MultiFieldEntityT<TEntity extends IEntity>} value
 * @param {number} entitiesCountLimit
 * @returns {TEntity[]}
 */
exports.asOrderedMultiFieldEntities = function (value, entitiesCountLimit) {
    var result = array_1.makeArray(entitiesCountLimit);
    var multiFieldEntities = exports.asMultiFieldEntities(value);
    if (Array.isArray(multiFieldEntities)) {
        var cursor_1 = 0;
        result.forEach(function (_, index) {
            var entity = multiFieldEntities[index];
            if (!R.isNil(entity)) {
                if (entity.newEntity) {
                    result[entity.index] = entity;
                }
                else {
                    result[cursor_1++] = entity;
                }
            }
        });
    }
    return result;
};
/**
 * @stable [14.10.2019]
 * @param {MultiFieldEntityT<TEntity extends IEntity>} multiFieldEntity
 * @param {(entity: TEntity, index: number) => TResult} mapper
 * @returns {TResult[]}
 */
exports.asMultiFieldMappedEntities = function (multiFieldEntity, mapper) {
    return cond_1.ifNotNilThanValue(exports.asMultiFieldEntities(multiFieldEntity), function (result) { return result.map(mapper); }, definitions_interface_1.UNDEF_SYMBOL);
};
/**
 * @stable [14.10.2019]
 * @param {MultiFieldEntityT<TEntity extends IEntity> | EntityIdT[]} multiFieldEntity
 * @returns {EntityIdT[]}
 */
exports.asMultiFieldMappedEntitiesIds = function (multiFieldEntity) {
    return exports.asMultiFieldMappedEntities(multiFieldEntity, function (entity) { return entity.id; });
};
//# sourceMappingURL=field.js.map